package com.example.flutter_bl_umpush;

import android.app.Notification;
import android.content.Context;
import android.os.Handler;
import android.util.Log;
import android.util.SparseArray;
import android.widget.Toast;

import com.umeng.analytics.MobclickAgent;
import com.umeng.commonsdk.UMConfigure;
import com.umeng.message.IUmengCallback;
import com.umeng.message.IUmengRegisterCallback;
import com.umeng.message.MsgConstant;
import com.umeng.message.PushAgent;
import com.umeng.message.UTrack;
import com.umeng.message.UmengMessageHandler;
import com.umeng.message.UmengNotificationClickHandler;
import com.umeng.message.common.inter.ITagManager;
import com.umeng.message.entity.UMessage;
import com.umeng.message.tag.TagManager;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

import static android.os.Looper.getMainLooper;

/**
 * FlutterBlUmpushPlugin
 */
public class FlutterBlUmpushPlugin implements MethodCallHandler {

    private static String TAG = "FlutterBlUmpushPlugin";
    // 通道
    public static MethodChannel channel;
    private static Registrar registrar;
    private Context mContext;

    private static SparseArray<Result> sCacheMap = new SparseArray<>();

    /**
     * Plugin registration.
     */
    public static void registerWith(Registrar registrar) {

        final MethodChannel methodChannel = new MethodChannel(registrar.messenger(), "flutter_bl_umpush");
        methodChannel.setMethodCallHandler(new FlutterBlUmpushPlugin());
        channel = methodChannel;
        // 用于获取Context activity
        FlutterBlUmpushPlugin.registrar = registrar;
    }

    @Override
    public void onMethodCall(MethodCall call, Result result) {

        switch (call.method) {
            case "getPlatformVersion":
                result.success("Android " + android.os.Build.VERSION.RELEASE);
                break;
            case "initWithAppkey":
                init(call, result);
                break;
            case "addAlias":
                addAlias((String) call.argument("account"), (String) call.argument("type"), result);
                break;
            case "addtags":
                addTags((List) call.arguments(), result);
                break;
            case "delAlias":
                deleteAlias((String) call.argument("account"), (String) call.argument("type"), result);
                break;
            case "setScenarioType":
                setScenarioType(call, result);
                break;
            case "umengEnterViewWithName":
                pageStart(call, result);
                break;
            case "umengOutViewWithName":
                pageEnd(call, result);
                break;
            case "umengEventCountWithId":
                eventCounts(call, result);
                break;
            case "umengSignIn":
                umSignIn(call, result);
                break;
            case "umengSignOff":
                umSignOff(call, result);
                break;
            default:
                result.notImplemented();
        }
    }

    private Context getCurrentActivity() {

        return registrar.activity();
    }

    // 初始化
    private void init(MethodCall call, Result result) {
        mContext = getCurrentActivity();
        initUmeng(mContext, PushAgent.getInstance(mContext), call, result);
        Log.i(TAG, "init Push Success!");
    }

    private void initUmeng(final Context context, PushAgent mPushAgent, MethodCall call, Result result) {

        UMConfigure.setLogEnabled(false);
        UMConfigure.init(context, (String) call.argument("appKey"),
                "Umeng", UMConfigure.DEVICE_TYPE_PHONE, (String) call.argument("secret"));

        //统计SDK是否支持采集在子进程中打点的自定义事件，默认不支持
        UMConfigure.setProcessEvent(true);//支持多进程打点
        //sdk开启通知声音
        mPushAgent.setNotificationPlaySound(MsgConstant.NOTIFICATION_PLAY_SDK_ENABLE);

    }

    private void addTags(final List tags, Result result) {
        final int sequence = getSequence();
        Log.i(TAG, "tags to be added: " + tags.toString() + " sequence: " + sequence);
        sCacheMap.put(sequence, result);
        PushAgent.getInstance(getCurrentActivity()).getTagManager().addTags(new TagManager.TCallBack() {
            @Override
            public void onMessage(boolean isSuccess, ITagManager.Result result) {
            }
        }, getStringArrary(tags));
    }

    private void addAlias(String alias, String aliasType, Result result) {
        final int sequence = getSequence();
        Log.i(TAG, "Add alias, sequence: " + sequence);
        sCacheMap.put(sequence, result);
        PushAgent.getInstance(getCurrentActivity()).addAlias(alias, aliasType, new UTrack.ICallBack() {
            @Override
            public void onMessage(boolean isSuccess, String s) {
            }
        });
    }

    private void setAlias(String alias, String aliasType, Result result) {
        final int sequence = getSequence();
        Log.i(TAG, "Set alias, sequence: " + sequence);
        sCacheMap.put(sequence, result);
        PushAgent.getInstance(getCurrentActivity()).setAlias(alias, aliasType, new UTrack.ICallBack() {
            @Override
            public void onMessage(boolean isSuccess, String s) {
            }
        });
    }

    private void deleteAlias(String alias, String aliasType, Result result) {
        final int sequence = getSequence();
        Log.i(TAG, "Delete alias, sequence: " + sequence);
        sCacheMap.put(sequence, result);
        PushAgent.getInstance(getCurrentActivity()).deleteAlias(alias, aliasType, new UTrack.ICallBack() {
            @Override
            public void onMessage(boolean isSuccess, String s) {
            }
        });
    }

    private void getAlias(Result result) {
        result.notImplemented();
    }

    // 登陆统计
    private void umSignIn(MethodCall call, Result result) {
        MobclickAgent.onProfileSignIn((String) call.arguments());
        //当用户使用第三方账号（如新浪微博）登录时，可以这样统计：
//        MobclickAgent.onProfileSignIn("WB","userID");
        // Session间隔时长,单位是毫秒，默认Session间隔时间是30秒,一般情况下不用修改此值
//        MobclickAgent.setSessionContinueMillis((Long) call.arguments());
        result.success(null);
    }

    // 登出
    private void umSignOff(MethodCall call, Result result) {
        MobclickAgent.onProfileSignOff();
        result.success(null);
    }

    // 打开页面
    private void pageStart(MethodCall call, Result result) {
        MobclickAgent.onPageStart((String) call.arguments());
        MobclickAgent.onResume(mContext);
        result.success(null);
    }

    // 关闭页面
    private void pageEnd(MethodCall call, Result result) {
        MobclickAgent.onPageEnd((String) call.arguments());
        MobclickAgent.onPause(mContext);
        result.success(null);
    }

    // 统计事件行为次数
    private void eventCounts(MethodCall call, Result result) {
        MobclickAgent.onEvent(mContext, (String) call.arguments());
        result.success(null);
    }

    // 场景
    private void setScenarioType(MethodCall call, Result result) {
        //EScenarioType.E_UM_NORMAL 普通统计场景，如果您在埋点过程中没有使用到
        //U-Game统计接口，请使用普通统计场景。
        //EScenarioType.E_UM_GAME 游戏场景 ，如果您在埋点过程中需要使用到U-Game
        //统计接口，则必须设置游戏场景，否则所有的U-Game统计接口不会生效。
        MobclickAgent.setScenarioType(mContext, MobclickAgent.EScenarioType.E_UM_NORMAL);
        result.success(null);
    }

    private void setLatestNotificationNumber(int number) {
        try {
            mContext = getCurrentActivity();
            PushAgent.getInstance(getCurrentActivity()).setDisplayNotificationNumber(number);
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    private void setSilenceTime(Map map) {
        try {
            mContext = getCurrentActivity();
            List array = (List) map.get("days");
            Set<Integer> days = new HashSet<Integer>();
            for (int i = 0; i < array.size(); i++) {
                days.add((Integer) array.get(i));
            }
            int startHour = (Integer) map.get("startHour");
            int endHour = (Integer) map.get("endHour");
            PushAgent.getInstance(getCurrentActivity()).setNoDisturbMode(startHour, 0, endHour, 0);
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    private void setResourcePackageName(Map map) {
        try {
            mContext = getCurrentActivity();
            String packageName = (String) map.get("packageName");
            PushAgent.getInstance(mContext).setResourcePackageName(packageName);
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    private void stopPush() {
        mContext = getCurrentActivity();
        PushAgent.getInstance(mContext).disable(
                new IUmengCallback() {
                    @Override
                    public void onSuccess() {
                    }

                    @Override
                    public void onFailure(String s, String s1) {
                        Log.i(TAG, String.format("UMPush disable Failure. %s, %s", s, s1));
                    }
                }
        );
        Log.i(TAG, "Stop push");
    }

    private void resumePush() {
        mContext = getCurrentActivity();
        PushAgent.getInstance(mContext).enable(
                new IUmengCallback() {
                    @Override
                    public void onSuccess() {
                    }

                    @Override
                    public void onFailure(String s, String s1) {
                        Log.i(TAG, String.format("UMPush enable Failure. %s, %s", s, s1));
                    }
                }
        );
        Log.i(TAG, "Resume push");
    }


    private int getSequence() {
        SimpleDateFormat sdf = new SimpleDateFormat("MMddHHmmss");
        String date = sdf.format(new Date());
        return Integer.valueOf(date);
    }

    private String[] getStringArrary(List list) {
        String[] v = new String[list.size()];
        for (int i = 0; i < list.size(); i++) {
            v[i] = list.get(i).toString();
        }
        return v;
    }
}
