import 'dart:async';

import 'package:flutter/services.dart';

class FlutterBlUmpushPlugin {
  static const MethodChannel _channel =
  const MethodChannel('flutter_bl_umpush');

  ///接收推送消息
  static StreamController<Map> _receivePushListener =
  new StreamController.broadcast();
  ///打开推送消息
  static StreamController<Map> _recvOpenNotificationListener =
  new StreamController.broadcast();
  ///自定义消息（暂时）
  static StreamController<Map> _recvCustomMsgController =
  new StreamController.broadcast();

  ///自定义消息（暂时）
  static StreamController<Map> _recvDeviceTokenController =
  new StreamController.broadcast();

  static Map<Function, StreamSubscription> listeners = {};


  /**
   * 监听：接收推送事件
   */
  static void addDeviceTokenListener(void onData(Map  notification)) {
    listeners[onData] = _recvDeviceTokenController.stream.listen(onData);
  }

  static void removeRecvDeviceTokenListener(void onData(Map deviceToken)) {
    removeListener(onData);
  }

  /**
   * 监听：接收推送事件
   */
  static void addReceiveNotificationListener(void onData(Map notification)) {
    listeners[onData] = _receivePushListener.stream.listen(onData);
  }

  /**
   * 取消监听：接收推送事件
   */
  static void removeReceiveNotificationListener(void onData(Map notification)) {
    removeListener(onData);
  }

  /**
   * 监听：点击推送事件
   */
  static void addReceiveOpenNotificationListener(
      void onData(Map notification)) {
    listeners[onData] = _recvOpenNotificationListener.stream.listen(onData);
  }

  /**
   * 取消监听：点击推送事件
   */
  static void removeReceiveOpenNotificationListener(
      void onData(Map notification)) {
    removeListener(onData);
  }

  /**
   * 监听：自定义消息后事件
   */
  static void addReceiveCustomMsgListener(void onData(Map data)) {
    listeners[onData] = _recvCustomMsgController.stream.listen(onData);
  }

  /**
   * 取消监听：自定义消息后事件
   */
  static void removeReceiveCustomMsgListener(void onData(Map data)) {
    removeListener(onData);
  }

  static void removeListener(void onData(dynamic data)) {
    StreamSubscription listener = listeners[onData];
    if (listener == null) return;
    listener.cancel();
    listeners.remove(onData);
  }

  static Future<dynamic> _handler(MethodCall call) {
    //print("handle mehtod call ${call.method} ${call.arguments}");
    String method = call.method;
    switch (method) {
      case 'receivePushMsg':
        {
          var map = call.arguments;
          _receivePushListener.add(map);
        }
        break;
      case 'openNotification':
        {
          dynamic dic = call.arguments;
          _recvOpenNotificationListener.add(dic);

          print("openNotification --> arguments: ");
          print(call.arguments);
        }
        break;
      case 'receiveNotification':
        {
          dynamic map = call.arguments;
          _recvCustomMsgController.add(map);
        }
        break;
      case 'callbackDeviceToken':
        {
          dynamic token = call.arguments;
          print("callbackDeviceToken --> arguments: " + token.toString());

          dynamic dic = {"deviceToken":token};
          if(token!=null){
            _recvDeviceTokenController.add(dic);
            print("callbackDeviceToken --> token: " + token.toString());
          }
        }
        break;
    }
    return new Future.value(null);
  }

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  //初始化um推送
  static Future<bool> initWithAppkey(
      String appKey, String secret, String channel) async {
    _channel.setMethodCallHandler(_handler);
    final bool result = await _channel.invokeMethod('initWithAppkey',
        {"appKey": appKey, "secret": secret, "channel": channel});
    return result;
  }

  //初始化um推送
  static Future<bool> initUMPushOptions(String appKey, String secret, String channel) async {
//    _channel.setMethodCallHandler(_handler);
    final bool result = await _channel.invokeMethod('initUMPushOptions',
        {"appKey": appKey, "secret": secret, "channel": channel});
    return result;
  }

  //设置别名
  static Future<bool> addAlias(String account, String type) async {
    final bool result = await _channel.invokeMethod('addAlias', {'account':account,'type':type});
    return result;
  }

  //删除别名
  static Future<bool> delAlias(String account, String type) async {
    final bool result = await _channel.invokeMethod("delAlias", {'account':account,'type':type});
    return result;
  }

  //设置标签
  static Future<bool> addtags(List tag) async {
    final bool result = await _channel.invokeMethod('addtags', tag);
    return result;
  }

  //设置标签
  static Future<bool> setBadgeClear(bool isClear) async {
    final bool result = await _channel.invokeMethod('setBadgeClear', isClear);
    return result;
  }

  //设置标签
  static Future<bool> setAutoAlert(bool isAlert) async {
    final bool result = await _channel.invokeMethod('setAutoAlert', isAlert);
    return result;
  }

  //启动统计 场景
  static Future<bool> setScenarioType() async {
    final bool result = await _channel.invokeMethod('setScenarioType');
  }

//页面统计 - 进入
  static Future<bool> umengEnterViewWithName(String name) async {
    final bool result =
    await _channel.invokeMethod('umengEnterViewWithName', name);
    return result;
  }

//页面统计 - 退出
  static Future<bool> umengOutViewWithName(String name) async {
    final bool result =
    await _channel.invokeMethod('umengOutViewWithName', name);
    return result;
  }

//计数事件统计
  static Future<bool> umengEventCountWithId(String eventId) async {
    final bool result =
    await _channel.invokeMethod('umengEventCountWithId', eventId);
    return result;
  }

  //登录
  static Future<bool> umengSignIn(String userId) async {
    final bool result = await _channel.invokeMethod("umengSignIn", userId);
    return result;
  }

  //登出
  static Future<bool> umengSignOff() async {
    final bool result = await _channel.invokeMethod("umengSignOff");
    return result;
  }
}
